@@ -11,7 +11,7 @@
 
 #include "atoms.h"
 #include "core/output.h"
-#include "core/platform.h"
+#include "core/outputbackend.h"
 #include "cursor.h"
 #include "deleted.h"
 #include "effects.h"
@@ -82,8 +82,8 @@ void MoveResizeWindowTest::initTestCase()
     qRegisterMetaType<KWin::Deleted *>();
     qRegisterMetaType<KWin::MaximizeMode>("MaximizeMode");
     QSignalSpy applicationStartedSpy(kwinApp(), &Application::started);
-    kwinApp()->platform()->setInitialWindowSize(QSize(1280, 1024));
     QVERIFY(waylandServer()->init(s_socketName));
+    QMetaObject::invokeMethod(kwinApp()->outputBackend(), "setVirtualOutputs", Qt::DirectConnection, Q_ARG(QVector<QRect>, QVector<QRect>() << QRect(0, 0, 1280, 1024)));
     kwinApp()->start();
     QVERIFY(applicationStartedSpy.wait());
     const auto outputs = workspace()->outputs();
@@ -108,8 +108,6 @@ void MoveResizeWindowTest::cleanup()
 
 void MoveResizeWindowTest::testMove()
 {
-    using namespace KWayland::Client;
-
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
@@ -181,7 +179,6 @@ void MoveResizeWindowTest::testMove()
 void MoveResizeWindowTest::testResize()
 {
     // a test case which manually resizes a window
-    using namespace KWayland::Client;
 
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
@@ -315,8 +312,6 @@ void MoveResizeWindowTest::testPackTo_data()
 
 void MoveResizeWindowTest::testPackTo()
 {
-    using namespace KWayland::Client;
-
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
@@ -353,8 +348,6 @@ void MoveResizeWindowTest::testPackAgainstClient_data()
 
 void MoveResizeWindowTest::testPackAgainstClient()
 {
-    using namespace KWayland::Client;
-
     std::unique_ptr<KWayland::Client::Surface> surface1(Test::createSurface());
     QVERIFY(surface1 != nullptr);
     std::unique_ptr<KWayland::Client::Surface> surface2(Test::createSurface());
@@ -420,8 +413,6 @@ void MoveResizeWindowTest::testGrowShrink_data()
 
 void MoveResizeWindowTest::testGrowShrink()
 {
-    using namespace KWayland::Client;
-
     // block geometry helper
     std::unique_ptr<KWayland::Client::Surface> surface1(Test::createSurface());
     QVERIFY(surface1 != nullptr);
@@ -480,7 +471,6 @@ void MoveResizeWindowTest::testPointerMoveEnd_data()
 void MoveResizeWindowTest::testPointerMoveEnd()
 {
     // this test verifies that moving a window through pointer only ends if all buttons are released
-    using namespace KWayland::Client;
 
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
@@ -518,12 +508,11 @@ void MoveResizeWindowTest::testPointerMoveEnd()
 }
 void MoveResizeWindowTest::testClientSideMove()
 {
-    using namespace KWayland::Client;
     Cursors::self()->mouse()->setPos(640, 512);
-    std::unique_ptr<Pointer> pointer(Test::waylandSeat()->createPointer());
-    QSignalSpy pointerEnteredSpy(pointer.get(), &Pointer::entered);
-    QSignalSpy pointerLeftSpy(pointer.get(), &Pointer::left);
-    QSignalSpy buttonSpy(pointer.get(), &Pointer::buttonStateChanged);
+    std::unique_ptr<KWayland::Client::Pointer> pointer(Test::waylandSeat()->createPointer());
+    QSignalSpy pointerEnteredSpy(pointer.get(), &KWayland::Client::Pointer::entered);
+    QSignalSpy pointerLeftSpy(pointer.get(), &KWayland::Client::Pointer::left);
+    QSignalSpy buttonSpy(pointer.get(), &KWayland::Client::Pointer::buttonStateChanged);
 
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     std::unique_ptr<Test::XdgToplevel> shellSurface(Test::createXdgToplevelSurface(surface.get()));
@@ -577,14 +566,13 @@ void MoveResizeWindowTest::testPlasmaShellSurfaceMovable_data()
 void MoveResizeWindowTest::testPlasmaShellSurfaceMovable()
 {
     // this test verifies that certain window types from PlasmaShellSurface are not moveable or resizable
-    using namespace KWayland::Client;
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
     std::unique_ptr<Test::XdgToplevel> shellSurface(Test::createXdgToplevelSurface(surface.get()));
     QVERIFY(shellSurface != nullptr);
     // and a PlasmaShellSurface
-    std::unique_ptr<PlasmaShellSurface> plasmaSurface(Test::waylandPlasmaShell()->createSurface(surface.get()));
+    std::unique_ptr<KWayland::Client::PlasmaShellSurface> plasmaSurface(Test::waylandPlasmaShell()->createSurface(surface.get()));
     QVERIFY(plasmaSurface != nullptr);
     QFETCH(KWayland::Client::PlasmaShellSurface::Role, role);
     plasmaSurface->setRole(role);
@@ -722,7 +710,6 @@ void MoveResizeWindowTest::testAdjustClientGeometryOfAutohidingX11Panel()
     QVERIFY(panel->isDock());
 
     // let's create a window
-    using namespace KWayland::Client;
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
@@ -783,15 +770,14 @@ void MoveResizeWindowTest::testAdjustClientGeometryOfAutohidingWaylandPanel()
     // see BUG 365892
 
     // first create our panel
-    using namespace KWayland::Client;
     std::unique_ptr<KWayland::Client::Surface> panelSurface(Test::createSurface());
     QVERIFY(panelSurface != nullptr);
     std::unique_ptr<Test::XdgToplevel> panelShellSurface(Test::createXdgToplevelSurface(panelSurface.get()));
     QVERIFY(panelShellSurface != nullptr);
-    std::unique_ptr<PlasmaShellSurface> plasmaSurface(Test::waylandPlasmaShell()->createSurface(panelSurface.get()));
+    std::unique_ptr<KWayland::Client::PlasmaShellSurface> plasmaSurface(Test::waylandPlasmaShell()->createSurface(panelSurface.get()));
     QVERIFY(plasmaSurface != nullptr);
-    plasmaSurface->setRole(PlasmaShellSurface::Role::Panel);
-    plasmaSurface->setPanelBehavior(PlasmaShellSurface::PanelBehavior::AutoHide);
+    plasmaSurface->setRole(KWayland::Client::PlasmaShellSurface::Role::Panel);
+    plasmaSurface->setPanelBehavior(KWayland::Client::PlasmaShellSurface::PanelBehavior::AutoHide);
     QFETCH(QRect, panelGeometry);
     plasmaSurface->setPosition(panelGeometry.topLeft());
     // let's render
@@ -854,8 +840,6 @@ void MoveResizeWindowTest::testResizeForVirtualKeyboard_data()
 
 void MoveResizeWindowTest::testResizeForVirtualKeyboard()
 {
-    using namespace KWayland::Client;
-
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
@@ -929,8 +913,6 @@ void MoveResizeWindowTest::testResizeForVirtualKeyboard()
 
 void MoveResizeWindowTest::testResizeForVirtualKeyboardWithMaximize()
 {
-    using namespace KWayland::Client;
-
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
@@ -979,8 +961,6 @@ void MoveResizeWindowTest::testResizeForVirtualKeyboardWithMaximize()
 
 void MoveResizeWindowTest::testResizeForVirtualKeyboardWithFullScreen()
 {
-    using namespace KWayland::Client;
-
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
 
@@ -1032,7 +1012,6 @@ void MoveResizeWindowTest::testDestroyMoveClient()
     // the associated client is destroyed.
 
     // Create the test client.
-    using namespace KWayland::Client;
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
     std::unique_ptr<Test::XdgToplevel> shellSurface(Test::createXdgToplevelSurface(surface.get()));
@@ -1067,7 +1046,6 @@ void MoveResizeWindowTest::testDestroyResizeClient()
     // the associated client is destroyed.
 
     // Create the test client.
-    using namespace KWayland::Client;
     std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
     QVERIFY(surface != nullptr);
     std::unique_ptr<Test::XdgToplevel> shellSurface(Test::createXdgToplevelSurface(surface.get()));
