@@ -6,28 +6,21 @@
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
-#ifndef KWIN_DRM_OUTPUT_H
-#define KWIN_DRM_OUTPUT_H
+#pragma once
 
 #include "drm_abstract_output.h"
 #include "drm_object.h"
-#include "drm_object_plane.h"
+#include "drm_plane.h"
 
 #include <QObject>
 #include <QPoint>
+#include <QPointer>
 #include <QSize>
 #include <QTimer>
 #include <QVector>
 #include <chrono>
 #include <xf86drmMode.h>
 
-namespace KWaylandServer
-{
-class DrmLeaseConnectorV1Interface;
-class DrmLeaseDeviceV1Interface;
-class DrmLeaseV1Interface;
-}
-
 namespace KWin
 {
 
@@ -35,21 +28,20 @@ class DrmConnector;
 class DrmGpu;
 class DrmPipeline;
 class DumbSwapchain;
-class GLTexture;
-class RenderTarget;
+class DrmLease;
 
 class KWIN_EXPORT DrmOutput : public DrmAbstractOutput
 {
     Q_OBJECT
 public:
-    DrmOutput(const std::shared_ptr<DrmConnector> &connector, KWaylandServer::DrmLeaseDeviceV1Interface *leaseDevice);
+    DrmOutput(const std::shared_ptr<DrmConnector> &connector);
     ~DrmOutput() override;
 
     DrmConnector *connector() const;
     DrmPipeline *pipeline() const;
 
     bool present() override;
-    DrmOutputLayer *outputLayer() const override;
+    DrmOutputLayer *primaryLayer() const override;
 
     bool queueChanges(const OutputConfiguration &config);
     void applyQueuedChanges(const OutputConfiguration &config);
@@ -57,13 +49,12 @@ public:
     void updateModes();
     void updateDpmsMode(DpmsMode dpmsMode);
 
-    bool usesSoftwareCursor() const override;
-    void updateCursor();
-    void moveCursor();
+    bool setCursor(CursorSource *source) override;
+    bool moveCursor(const QPoint &position) override;
 
-    KWaylandServer::DrmLeaseV1Interface *lease() const;
+    DrmLease *lease() const;
     bool addLeaseObjects(QVector<uint32_t> &objectList);
-    void leased(KWaylandServer::DrmLeaseV1Interface *lease);
+    void leased(DrmLease *lease);
     void leaseEnded();
 
     void setColorTransformation(const std::shared_ptr<ColorTransformation> &transformation) override;
@@ -74,23 +65,20 @@ private:
 
     QList<std::shared_ptr<OutputMode>> getModes() const;
 
-    void renderCursorOpengl(const RenderTarget &renderTarget, const QSize &cursorSize);
-    void renderCursorQPainter(const RenderTarget &renderTarget);
-
     DrmPipeline *m_pipeline;
     const std::shared_ptr<DrmConnector> m_connector;
 
     bool m_setCursorSuccessful = false;
     bool m_moveCursorSuccessful = false;
-    bool m_cursorTextureDirty = true;
-    std::unique_ptr<GLTexture> m_cursorTexture;
     QTimer m_turnOffTimer;
-    std::unique_ptr<KWaylandServer::DrmLeaseConnectorV1Interface> m_offer;
-    KWaylandServer::DrmLeaseV1Interface *m_lease = nullptr;
+    DrmLease *m_lease = nullptr;
+
+    struct {
+        QPointer<CursorSource> source;
+        QPoint position;
+    } m_cursor;
 };
 
 }
 
 Q_DECLARE_METATYPE(KWin::DrmOutput *)
-
-#endif
