@@ -7,14 +7,14 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef KWIN_GLPLATFORM_H
-#define KWIN_GLPLATFORM_H
+#pragma once
 
-#include <kwinglutils_export.h>
 #include <kwinglobals.h>
+#include <kwinglutils_export.h>
 
 #include <QByteArray>
 #include <QSet>
+#include <memory>
 
 namespace KWin
 {
@@ -80,7 +80,7 @@ enum GLFeature {
 };
 
 enum Driver {
-    Driver_R100,  // Technically "Radeon"
+    Driver_R100, // Technically "Radeon"
     Driver_R200,
     Driver_R300C,
     Driver_R300G,
@@ -173,7 +173,6 @@ enum ChipClass {
 };
 // clang-format on
 
-
 class KWINGLUTILS_EXPORT GLPlatform
 {
 public:
@@ -314,7 +313,7 @@ public:
      **/
     bool isVirgl() const;
 
-     /**
+    /**
      * @returns @c true if the "GPU" is a Panfrost Mali GPU
      * @since 5.21.5
      **/
@@ -444,28 +443,25 @@ private:
     qint64 m_galliumVersion;
     qint64 m_serverVersion;
     qint64 m_kernelVersion;
-    bool m_looseBinding: 1;
-    bool m_supportsGLSL: 1;
-    bool m_limitedGLSL: 1;
-    bool m_textureNPOT: 1;
-    bool m_limitedNPOT: 1;
-    bool m_packInvert: 1;
-    bool m_virtualMachine: 1;
-    bool m_preferBufferSubData: 1;
+    bool m_looseBinding : 1;
+    bool m_supportsGLSL : 1;
+    bool m_limitedGLSL : 1;
+    bool m_textureNPOT : 1;
+    bool m_limitedNPOT : 1;
+    bool m_packInvert : 1;
+    bool m_virtualMachine : 1;
+    bool m_preferBufferSubData : 1;
     OpenGLPlatformInterface m_platformInterface;
-    bool m_gles: 1;
-    static GLPlatform *s_platform;
+    bool m_gles : 1;
+    static std::unique_ptr<GLPlatform> s_platform;
 };
 
 inline GLPlatform *GLPlatform::instance()
 {
-    if (!s_platform)
-        s_platform = new GLPlatform;
-
-    return s_platform;
+    if (!s_platform) {
+        s_platform.reset(new GLPlatform());
+    }
+    return s_platform.get();
 }
 
 } // namespace KWin
-
-#endif // KWIN_GLPLATFORM_H
-
