@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef KWIN_RULES_H
-#define KWIN_RULES_H
+#pragma once
 
 #include <QRectF>
 #include <QVector>
@@ -192,10 +191,10 @@ public:
 private:
 #endif
     bool matchType(NET::WindowType match_type) const;
-    bool matchWMClass(const QByteArray &match_class, const QByteArray &match_name) const;
-    bool matchRole(const QByteArray &match_role) const;
+    bool matchWMClass(const QString &match_class, const QString &match_name) const;
+    bool matchRole(const QString &match_role) const;
     bool matchTitle(const QString &match_title) const;
-    bool matchClientMachine(const QByteArray &match_machine, bool local) const;
+    bool matchClientMachine(const QString &match_machine, bool local) const;
 #ifdef KCMRULES
 private:
 #endif
@@ -210,14 +209,14 @@ private:
 #endif
     int temporary_state; // e.g. for kstart
     QString description;
-    QByteArray wmclass;
+    QString wmclass;
     StringMatch wmclassmatch;
     bool wmclasscomplete;
-    QByteArray windowrole;
+    QString windowrole;
     StringMatch windowrolematch;
     QString title;
     StringMatch titlematch;
-    QByteArray clientmachine;
+    QString clientmachine;
     StringMatch clientmachinematch;
     NET::WindowTypes types; // types for matching
     PlacementPolicy placement;
@@ -388,5 +387,3 @@ QDebug &operator<<(QDebug &stream, const Rules *);
 } // namespace
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KWin::Rules::Types)
-
-#endif
