@@ -11,6 +11,7 @@
 
 namespace KWaylandServer
 {
+class ClientConnection;
 class SeatInterface;
 class SurfaceInterface;
 class KeyboardInterfacePrivate;
@@ -54,6 +55,7 @@ public:
     void setRepeatInfo(qint32 charactersPerSecond, qint32 delay);
 
     void sendKey(quint32 key, KeyboardKeyState state);
+    void sendKey(quint32 key, KeyboardKeyState state, ClientConnection *client);
     void sendModifiers(quint32 depressed, quint32 latched, quint32 locked, quint32 group);
 
 private:
